/* This file was generated by syncqt. */
#ifndef QT_QTCONNECTIVITY_VERSION_H
#define QT_QTCONNECTIVITY_VERSION_H

#define QTCONNECTIVITY_VERSION_STR "6.10.2"

#define QTCONNECTIVITY_VERSION 0x0601002

#endif // QT_QTCONNECTIVITY_VERSION_H
